@@ -24,7 +24,6 @@
 #include <gtk/gtk.h>
 #include <glib.h>
 #include "debug.h"
-#include "vtebg.h"
 #include "vtedraw.h"
 #include "vte-private.h"
 
@@ -589,7 +588,6 @@ font_info_find_for_context (PangoContext *context)
 static struct font_info *
 font_info_create_for_context (PangoContext               *context,
 			      const PangoFontDescription *desc,
-			      VteTerminalAntiAlias        antialias,
 			      PangoLanguage              *language,
 			      guint                       fontconfig_timestamp)
 {
@@ -610,37 +608,16 @@ font_info_create_for_context (PangoContext               *context,
 
 	pango_context_set_language (context, language);
 
-	switch (antialias) {
-		cairo_font_options_t *font_options;
-		cairo_antialias_t cr_aa;
+        /* Make sure our contexts have a font_options set.  We use
+          * this invariant in our context hash and equal functions.
+          */
+        if (!pango_cairo_context_get_font_options (context)) {
+                cairo_font_options_t *font_options;
 
-	case VTE_ANTI_ALIAS_FORCE_ENABLE:
-	case VTE_ANTI_ALIAS_FORCE_DISABLE:
-
-		if (antialias == VTE_ANTI_ALIAS_FORCE_ENABLE)
-			cr_aa = CAIRO_ANTIALIAS_DEFAULT; /* let surface decide between gray and subpixel */
-		else
-			cr_aa = CAIRO_ANTIALIAS_NONE;
-
-		font_options = cairo_font_options_copy (pango_cairo_context_get_font_options (context));
-		cairo_font_options_set_antialias (font_options, cr_aa);
-		pango_cairo_context_set_font_options (context, font_options);
-		cairo_font_options_destroy (font_options);
-
-		break;
-
-	default:
-	case VTE_ANTI_ALIAS_USE_DEFAULT:
-		/* Make sure our contexts have a font_options set.  We use
-		 * this invariant in our context hash and equal functions.
-		 */
-		if (!pango_cairo_context_get_font_options (context)) {
-			font_options = cairo_font_options_create ();
-			pango_cairo_context_set_font_options (context, font_options);
-			cairo_font_options_destroy (font_options);
-		}
-		break;
-	}
+                font_options = cairo_font_options_create ();
+                pango_cairo_context_set_font_options (context, font_options);
+                cairo_font_options_destroy (font_options);
+        }
 
 	return font_info_find_for_context (context);
 }
@@ -648,25 +625,23 @@ font_info_create_for_context (PangoContext               *context,
 static struct font_info *
 font_info_create_for_screen (GdkScreen                  *screen,
 			     const PangoFontDescription *desc,
-			     VteTerminalAntiAlias        antialias,
 			     PangoLanguage              *language)
 {
 	GtkSettings *settings = gtk_settings_get_for_screen (screen);
 	int fontconfig_timestamp;
 	g_object_get (settings, "gtk-fontconfig-timestamp", &fontconfig_timestamp, NULL);
 	return font_info_create_for_context (gdk_pango_context_get_for_screen (screen),
-					     desc, antialias, language, fontconfig_timestamp);
+					     desc, language, fontconfig_timestamp);
 }
 
 static struct font_info *
 font_info_create_for_widget (GtkWidget                  *widget,
-			     const PangoFontDescription *desc,
-			     VteTerminalAntiAlias        antialias)
+			     const PangoFontDescription *desc)
 {
 	GdkScreen *screen = gtk_widget_get_screen (widget);
 	PangoLanguage *language = pango_context_get_language (gtk_widget_get_pango_context (widget));
 
-	return font_info_create_for_screen (screen, desc, antialias, language);
+	return font_info_create_for_screen (screen, desc, language);
 }
 
 static struct unistr_info *
@@ -759,8 +734,6 @@ guint _vte_draw_get_style(gboolean bold, gboolean italic) {
 struct _vte_draw {
 	GtkWidget *widget;
 
-	gint started;
-
 	struct font_info *fonts[4];
 	cairo_pattern_t *bg_pattern;
 
@@ -768,13 +741,12 @@ struct _vte_draw {
 };
 
 struct _vte_draw *
-_vte_draw_new (GtkWidget *widget)
+_vte_draw_new (void)
 {
 	struct _vte_draw *draw;
 
 	/* Create the structure. */
 	draw = g_slice_new0 (struct _vte_draw);
-	draw->widget = g_object_ref (widget);
 
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_new\n");
 
@@ -807,115 +779,35 @@ _vte_draw_free (struct _vte_draw *draw)
 		}
 	}
 
-	if (draw->widget != NULL) {
-		g_object_unref (draw->widget);
-	}
-
 	g_slice_free (struct _vte_draw, draw);
 }
 
 void
-_vte_draw_start (struct _vte_draw *draw)
+_vte_draw_set_cairo (struct _vte_draw *draw,
+                     cairo_t *cr)
 {
-	GdkWindow *window;
+        _vte_debug_print (VTE_DEBUG_DRAW, "%s cairo context\n", cr ? "Settings" : "Unsetting");
 
-	g_return_if_fail (gtk_widget_get_realized (draw->widget));
-
-	_vte_debug_print (VTE_DEBUG_DRAW, "draw_start\n");
-
-	if (draw->started == 0) {
-		window = gtk_widget_get_window(draw->widget);
-		g_object_ref (window);
-		draw->cr = gdk_cairo_create (window);
-	}
-
-	draw->started++;
-}
-
-void
-_vte_draw_end (struct _vte_draw *draw)
-{
-	g_return_if_fail (draw->started);
-
-	draw->started--;
-	if (draw->started == 0) {
-		cairo_destroy (draw->cr);
-		draw->cr = NULL;
-		g_object_unref (gtk_widget_get_window(draw->widget));
- 	}
-
-	_vte_debug_print (VTE_DEBUG_DRAW, "draw_end\n");
+        if (cr) {
+                g_assert (draw->cr == NULL);
+                draw->cr = cr;
+        } else {
+                g_assert (draw->cr != NULL);
+                draw->cr = NULL;
+        }
 }
 
 void
 _vte_draw_set_background_solid(struct _vte_draw *draw,
-			       double red,
-			       double green,
-			       double blue,
-			       double opacity)
+                               const GdkRGBA *color)
 {
 	if (draw->bg_pattern)
 		cairo_pattern_destroy (draw->bg_pattern);
 
-	draw->bg_pattern = cairo_pattern_create_rgba (red,
-						      green,
-						      blue,
-						      opacity);
-}
-
-void
-_vte_draw_set_background_image (struct _vte_draw *draw,
-			        VteBgSourceType type,
-			        GdkPixbuf *pixbuf,
-			        const char *filename,
-			        const PangoColor *color,
-			        double saturation)
-{
-	cairo_surface_t *surface;
-
-	/* Need a valid draw->cr for cairo_get_target () */
-	_vte_draw_start (draw);
-
-	surface = vte_bg_get_surface (vte_bg_get_for_screen (gtk_widget_get_screen (draw->widget)),
-				     type, pixbuf, filename,
-				     color, saturation,
-				     cairo_get_target(draw->cr));
-
-	_vte_draw_end (draw);
-
-	if (!surface)
-		return;
-
-	if (draw->bg_pattern)
-		cairo_pattern_destroy (draw->bg_pattern);
-
-	draw->bg_pattern = cairo_pattern_create_for_surface (surface);
-	cairo_surface_destroy (surface);
-	cairo_pattern_set_extend (draw->bg_pattern, CAIRO_EXTEND_REPEAT);
-}
-
-void
-_vte_draw_set_background_scroll (struct _vte_draw *draw,
-				 gint x, gint y)
-{
-	cairo_matrix_t matrix;
-
-	_vte_debug_print (VTE_DEBUG_DRAW,
-			"draw_set_scroll (%d, %d)\n",
-			x, y);
-
-	g_return_if_fail (draw->bg_pattern != NULL);
-
-	cairo_matrix_init_translate (&matrix, x, y);
-	cairo_pattern_set_matrix (draw->bg_pattern, &matrix);
-}
-
-void
-_vte_draw_clip (struct _vte_draw *draw, GdkRegion *region)
-{
-	_vte_debug_print (VTE_DEBUG_DRAW, "draw_clip\n");
-	gdk_cairo_region(draw->cr, region);
-	cairo_clip (draw->cr);
+	draw->bg_pattern = cairo_pattern_create_rgba (color->red,
+						      color->green,
+						      color->blue,
+						      color->alpha);
 }
 
 void
@@ -926,6 +818,7 @@ _vte_draw_clear (struct _vte_draw *draw, gint x, gint y, gint width, gint height
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_clear (%d, %d, %d, %d)\n",
 			  x,y,width, height);
 
+        g_assert(draw->cr);
 	cairo_rectangle (draw->cr, x, y, width, height);
 	cairo_set_operator (draw->cr, CAIRO_OPERATOR_SOURCE);
 	cairo_set_source (draw->cr, draw->bg_pattern);
@@ -934,16 +827,15 @@ _vte_draw_clear (struct _vte_draw *draw, gint x, gint y, gint width, gint height
 
 void
 _vte_draw_set_text_font (struct _vte_draw *draw,
-			const PangoFontDescription *fontdesc,
-			VteTerminalAntiAlias antialias)
+                         GtkWidget *widget,
+                         const PangoFontDescription *fontdesc)
 {
 	PangoFontDescription *bolddesc   = NULL;
 	PangoFontDescription *italicdesc = NULL;
 	PangoFontDescription *bolditalicdesc = NULL;
 	gint style, normal, bold, ratio;
 
-	_vte_debug_print (VTE_DEBUG_DRAW, "draw_set_text_font (aa=%d)\n",
-			  antialias);
+	_vte_debug_print (VTE_DEBUG_DRAW, "draw_set_text_font\n");
 
 	/* Free all fonts (make sure to destroy every font only once)*/
 	for (style = 3; style >= 0; style--) {
@@ -966,15 +858,11 @@ _vte_draw_set_text_font (struct _vte_draw *draw,
 	bolditalicdesc = pango_font_description_copy (bolddesc);
 	pango_font_description_set_style (bolditalicdesc, PANGO_STYLE_ITALIC);
 
-	draw->fonts[VTE_DRAW_NORMAL]  = font_info_create_for_widget (draw->widget,
-										fontdesc, antialias);
-	draw->fonts[VTE_DRAW_BOLD]    = font_info_create_for_widget (draw->widget,
-										bolddesc, antialias);
-	draw->fonts[VTE_DRAW_ITALIC]  = font_info_create_for_widget (draw->widget,
-										italicdesc, antialias);
+	draw->fonts[VTE_DRAW_NORMAL]  = font_info_create_for_widget (widget, fontdesc);
+	draw->fonts[VTE_DRAW_BOLD]    = font_info_create_for_widget (widget, bolddesc);
+	draw->fonts[VTE_DRAW_ITALIC]  = font_info_create_for_widget (widget, italicdesc);
 	draw->fonts[VTE_DRAW_ITALIC | VTE_DRAW_BOLD] =
-									font_info_create_for_widget (draw->widget,
-										bolditalicdesc, antialias);
+                font_info_create_for_widget (widget, bolditalicdesc);
 	pango_font_description_free (bolddesc);
 	pango_font_description_free (italicdesc);
 	pango_font_description_free (bolditalicdesc);
@@ -1040,6 +928,7 @@ _vte_draw_set_source_color_alpha (struct _vte_draw *draw,
                                   const PangoColor *color,
                                   guchar            alpha)
 {
+        g_assert(draw->cr);
 	cairo_set_source_rgba (draw->cr,
 			      color->red / 65535.,
 			      color->green / 65535.,
@@ -1060,6 +949,7 @@ _vte_draw_text_internal (struct _vte_draw *draw,
 
 	g_return_if_fail (font != NULL);
 
+        g_assert(draw->cr);
 	_vte_draw_set_source_color_alpha (draw, color, alpha);
 	cairo_set_operator (draw->cr, CAIRO_OPERATOR_OVER);
 
@@ -1118,7 +1008,7 @@ _vte_draw_text (struct _vte_draw *draw,
 	       struct _vte_draw_text_request *requests, gsize n_requests,
 	       const PangoColor *color, guchar alpha, guint style)
 {
-	g_return_if_fail (draw->started);
+        g_assert(draw->cr);
 
 	if (_vte_debug_on (VTE_DEBUG_DRAW)) {
 		GString *string = g_string_new ("");
@@ -1196,7 +1086,7 @@ _vte_draw_draw_rectangle (struct _vte_draw *draw,
 			 gint x, gint y, gint width, gint height,
 			 const PangoColor *color, guchar alpha)
 {
-	g_return_if_fail (draw->started);
+        g_assert(draw->cr);
 
 	_vte_debug_print (VTE_DEBUG_DRAW,
 			"draw_rectangle (%d, %d, %d, %d, color=(%d,%d,%d,%d))\n",
@@ -1216,7 +1106,7 @@ _vte_draw_fill_rectangle (struct _vte_draw *draw,
 			 gint x, gint y, gint width, gint height,
 			 const PangoColor *color, guchar alpha)
 {
-	g_return_if_fail (draw->started);
+        g_assert(draw->cr);
 
 	_vte_debug_print (VTE_DEBUG_DRAW,
 			"draw_fill_rectangle (%d, %d, %d, %d, color=(%d,%d,%d,%d))\n",
